@@ -7,8 +7,9 @@ use sphinx_lang::frontend;
 use sphinx_lang::BuildErrors;
 use sphinx_lang::source::{ModuleSource, SourceType, SourceText};
 use sphinx_lang::parser::stmt::{Stmt, StmtMeta};
-use sphinx_lang::codegen::Chunk;
+use sphinx_lang::codegen::{Program, CompiledProgram};
 use sphinx_lang::runtime::VirtualMachine;
+use sphinx_lang::runtime::module::{ModuleCache, GlobalEnv};
 use sphinx_lang::runtime::strings::StringInterner;
 use sphinx_lang::debug::symbol::BufferedResolver;
 
@@ -60,7 +61,11 @@ fn main() {
     }
     
     if module.is_none() {
-        start_repl(&args, version, None);
+        let mut module_cache = ModuleCache::new();
+        let repl_env = GlobalEnv::new();
+        
+        println!("\nSphinx Version {}\n", version);
+        Repl::new(&mut module_cache, &repl_env).run();
         return;
     }
     
@@ -72,33 +77,39 @@ fn main() {
     else if args.is_present("compile_only") {
         unimplemented!()
     }
+    else if args.is_present("interactive") {
+        if let Some(build) = build_program(&args, &module) {
+            let program = Program::load(build.program);
+            
+            let mut module_cache = ModuleCache::new();
+            let module_id = module_cache.insert(module, program.data);
+            
+            let repl_env = GlobalEnv::new();
+            
+            let mut vm = VirtualMachine::new_repl(&module_cache, &repl_env, module_id, &program.main);
+            vm.run().expect("runtime error");
+            
+            println!("\nSphinx Version {}\n", version);
+            Repl::new(&mut module_cache, &repl_env).run()
+        }
+    }
     else {
-        let exec_result = build_and_execute(&args, module);
-        
-        if let Ok(vm) = exec_result {
-            if args.is_present("interactive") {
-                start_repl(&args, version, Some(vm));
-            }
+        if let Some(build) = build_program(&args, &module) {
+            let program = Program::load(build.program);
+            
+            let mut module_cache = ModuleCache::new();
+            let module_id = module_cache.insert(module, program.data);
+            
+            let mut vm = VirtualMachine::new(&module_cache, module_id, &program.main);
+            vm.run().expect("runtime error");
         }
     }
 }
 
-fn start_repl(_args: &ArgMatches, version: &str, vm: Option<VirtualMachine>) {
-    println!("\nSphinx Version {}\n", version);
-    
-    let mut repl;
-    if let Some(vm) = vm {
-        repl = Repl::with_vm(vm);
-    } else {
-        repl = Repl::new();
-    }
-    
-    repl.run();
-}
 
-fn build_and_execute(_args: &ArgMatches, module: ModuleSource) -> Result<VirtualMachine, ()> {
+fn build_program<'m>(_args: &ArgMatches, source: &ModuleSource) -> Option<CompiledProgram> {
     // build module
-    let build_result = sphinx_lang::build_module(&module);
+    let build_result = sphinx_lang::build_module(source);
     if build_result.is_err() {
         match build_result.unwrap_err() {
             BuildErrors::Source(error) => {
@@ -106,25 +117,19 @@ fn build_and_execute(_args: &ArgMatches, module: ModuleSource) -> Result<Virtual
             }
             
             BuildErrors::Syntax(errors) => {
-                println!("Errors in file \"{}\":\n", module.name());
-                frontend::print_source_errors(&module, &errors);
+                println!("Errors in file \"{}\":\n", source.name());
+                frontend::print_source_errors(source, &errors);
             }
             
             BuildErrors::Compile(errors) => {
-                println!("Errors in file \"{}\":\n", module.name());
-                frontend::print_source_errors(&module, &errors);
+                println!("Errors in file \"{}\":\n", source.name());
+                frontend::print_source_errors(source, &errors);
             }
         }
-        return Err(());
+        return None;
     }
     
-    let program = build_result.unwrap();
-    let chunk = Chunk::load(program.bytecode);
-    let mut vm = VirtualMachine::new(chunk);
-    
-    vm.run().expect("runtime error");
-    
-    Ok(vm)
+    return Some(build_result.unwrap())
 }
 
 
@@ -157,8 +162,9 @@ fn parse_and_print_ast(_args: &ArgMatches, module: ModuleSource) {
 const PROMT_START: &str = ">>> ";
 const PROMT_CONTINUE: &str = "... ";
 
-struct Repl {
-    vm: Option<VirtualMachine>,
+struct Repl<'m> {
+    module_cache: &'m mut ModuleCache,
+    repl_env: &'m GlobalEnv,
 }
 
 enum ReadLine {
@@ -168,13 +174,11 @@ enum ReadLine {
     Quit,
 }
 
-impl Repl {
-    pub fn new() -> Self {
-        Self { vm: None }
-    }
-    
-    pub fn with_vm(vm: VirtualMachine) -> Self {
-        Self { vm: Some(vm) }
+impl<'m> Repl<'m> {
+    pub fn new(module_cache: &'m mut ModuleCache, repl_env: &'m GlobalEnv) -> Self {
+        Self {
+            module_cache, repl_env,
+        }
     }
     
     fn read_line(&self, prompt: &'static str) -> ReadLine {
@@ -267,8 +271,8 @@ impl Repl {
                 ast.push(StmtMeta::new(stmt, symbol))
             }
             
-            let program = match sphinx_lang::compile_ast(interner, ast) {
-                Ok(program) => program,
+            let build = match sphinx_lang::compile_ast(interner, ast) {
+                Ok(build) => build,
                 
                 Err(errors) => {
                     let resolver = BufferedResolver::new(input);
@@ -277,39 +281,16 @@ impl Repl {
                 }
             };
             
-            let chunk = Chunk::load(program.bytecode);
-            match self.vm {
-                Some(ref mut vm) => vm.reload_program(chunk),
-                None => { self.vm.replace(VirtualMachine::new(chunk)); },
-            }
+            let program = Program::load(build.program);
             
-            if let Err(error) = self.vm.as_mut().unwrap().run() {
+            let module_source = ModuleSource::new("<repl>", SourceType::String(input));
+            let module_id = self.module_cache.insert(module_source, program.data);
+            
+            let mut vm = VirtualMachine::new_repl(self.module_cache, self.repl_env, module_id, &program.main);
+            if let Err(error) = vm.run() {
                 println!("Runtime error: {:?}", error);
             }
             
-            // for stmt in stmts.iter() {
-            //     match stmt.variant() {
-            //         Stmt::Expression(expr) => {
-            //             let eval_ctx = EvalContext::new(&self.root_env);
-            //             let eval_result = eval_ctx.eval_expr(&expr);
-            //             log::debug!("{:?}", eval_result);
-            //             match eval_result {
-            //                 Ok(value) => {
-            //                     println!("{}", value.unwrap_value());
-            //                 },
-            //                 Err(error) => {
-            //                     println!("{:?}", error)
-            //                 },
-            //             }
-            //         },
-            //         _ => {
-            //             let exec_ctx = ExecContext::new(&self.root_env);
-            //             let exec_result = exec_ctx.exec(&stmt);
-            //             log::debug!("{:?}", exec_result);
-            //         },
-            //     }
-            // }
-            
         }
         
     }
