@@ -9,9 +9,12 @@ pub type CompileResult<T> = Result<T, CompileError>;
 
 #[derive(Debug)]
 pub enum ErrorKind {
+    ChunkCountLimit,
     ConstPoolLimit,
+    ParamCountLimit,
     TupleLengthLimit,
     LocalVariableLimit,
+    CalcJumpOffsetFailed,
     CantAssignImmutable,
     CantAssignNonLocal,
     Other(String),
@@ -36,7 +39,11 @@ impl CompileError {
     }
     
     pub fn with_symbol(mut self, symbol: DebugSymbol) -> Self {
-        self.symbol.replace(symbol); self 
+        self.symbol.get_or_insert(symbol); self 
+    }
+    
+    pub fn caused_by(mut self, error: impl Error + 'static) -> Self {
+        self.cause.replace(Box::new(error)); self
     }
     
     pub fn kind(&self) -> &ErrorKind { &self.kind }
@@ -62,9 +69,12 @@ impl fmt::Display for CompileError {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
         
         let message = match self.kind() {
+            ErrorKind::ChunkCountLimit => "chunk count limit reached",
             ErrorKind::ConstPoolLimit => "constant pool limit reached",
+            ErrorKind::ParamCountLimit => "parameter count limit exceeded",
             ErrorKind::TupleLengthLimit => "tuple length limit exceeded",
             ErrorKind::LocalVariableLimit => "local variable limit reached",
+            ErrorKind::CalcJumpOffsetFailed => "could not calculate jump offset",
             ErrorKind::CantAssignImmutable => "can't assign to immutable local variable",
             ErrorKind::CantAssignNonLocal => "can't assign to a non-local variable without the \"nonlocal\" keyword",
             ErrorKind::Other(message) => message,
