@@ -11,7 +11,10 @@ use std::mem::size_of;
                         // width set here so that the longest mnemonic is 16 chars
 const OP_NOP:           u8 = 0x00;
 const OP_RETURN:        u8 = 0x01;  // return from current function?
-const OP_EXIT:          u8 = 0x02;
+
+// yes, Nargs is both in the immediate stack and encoded in the call instruction
+// this bit of redundancy helps us deal with default and variadic arguments efficiently
+const OP_CALL:          u8 = 0x02;  // (u8); [ callobj Nargs arg_0 ... arg_n ] => [ ret_value ] 
 
 // 0x08-40        Immediate Values
 
@@ -19,6 +22,9 @@ const OP_POP:           u8 = 0x08;  // [ _ ] => []
 const OP_DROP:          u8 = 0x09;  // (u8); [ ... ] => []
 const OP_CLONE:         u8 = 0x0A;  // [ value ] => [ value value ]
 
+const OP_TUPLE:         u8 = 0x0B;  // (u8); [ ... ] => [ tuple ]
+const OP_TUPLEN:        u8 = 0x0C;  // [ ... N ] => [ tuple ]
+
 const OP_LD_CONST:      u8 = 0x10;  // (u8); _ => [ value ]
 const OP_LD_CONST_16:   u8 = 0x11;  // (u16); _ => [ value ]
 // const OP_LD_CONST_32:   u8 = 0x12;  // (u32); _ => [ value ]
@@ -35,8 +41,8 @@ const OP_ST_LOCAL_16:   u8 = 0x22;  // (u16); [ value ] => [ value ]
 // const OP_LD_LOCAL_32:   u8 = 0x23;  // (u32); _ => [ value ]
 const OP_LD_LOCAL:      u8 = 0x24;  // (u8);  _ => [ value ]
 const OP_LD_LOCAL_16:   u8 = 0x25;  // (u16); _ => [ value ]
-const OP_DP_LOCALS:     u8 = 0x26;  // (u8); [ ... ] => [] -- panics if there are any immediate values
 // const OP_LD_LOCAL_32:   u8 = 0x27;  // (u32); _ => [ value ]
+const OP_DP_LOCALS:     u8 = 0x26;  // (u8); [ ... ] => [] -- panics if there are any immediate values
 
 // const OP_LD_NAME:       u8 = 0x28;
 // const OP_LD_INDEX:      u8 = 0x29;
@@ -44,19 +50,19 @@ const OP_DP_LOCALS:     u8 = 0x26;  // (u8); [ ... ] => [] -- panics if there ar
 
 // Dynamic Insert/Store
 
-// const OP_IN_DYN         u8 = 0x30;  // [ value dyn_target bool ] => []
+// These are used to implement tuple-destructuring assignment/declaration
+// const OP_IN_DYN         u8 = 0x30;  // [ value dyn_target bool ] => [] 
 // const OP_ST_DYN         u8 = 0x31;  // [ value dyn_target ] => []
 
-const OP_NIL:           u8 = 0x38;  // _ => [ nil ]
-const OP_FALSE:         u8 = 0x39;  // _ => [ false ]
-const OP_TRUE:          u8 = 0x3A;  // _ => [ true ]
-const OP_EMPTY:         u8 = 0x3B;  // _ => [ () ]
-const OP_TUPLE:         u8 = 0x3C;  // (u8); [ ... ] => [ tuple ]
+const OP_LD_NIL:        u8 = 0x38;  // _ => [ nil ]
+const OP_LD_FALSE:      u8 = 0x39;  // _ => [ false ]
+const OP_LD_TRUE:       u8 = 0x3A;  // _ => [ true ]
+const OP_LD_EMPTY:      u8 = 0x3B;  // _ => [ () ]
 
 // small numbers
-const OP_U8:            u8 = 0x3D;  // (u8); _ => [ value ]
-const OP_I8:            u8 = 0x3E;  // (i8); _ => [ value ]
-const OP_F8:            u8 = 0x3F;  // (i8); _ => [ value ]
+const OP_LD_U8:         u8 = 0x3C;  // (u8); _ => [ value ]
+const OP_LD_I8:         u8 = 0x3D;  // (i8); _ => [ value ]
+const OP_LD_F8:         u8 = 0x3E;  // (i8); _ => [ value ]
 
 // const OP_DYN_TARGET:    u8 = 0x48;  // (u8); [ ... ] => [ dyn_target ]
 
@@ -88,21 +94,25 @@ const OP_LE:            u8 = 0x6B;
 const OP_GE:            u8 = 0x6C;
 const OP_GT:            u8 = 0x6D;
 
-// 0x70         Jumps
+// 0x70-7F      Jumps
 
 const OP_JUMP:          u8 = 0x70;  // (i16);
-const OP_JUMP_FALSE:    u8 = 0x71;  // (i16);
-const OP_JUMP_TRUE:     u8 = 0x72;  // (i16);
-// const OP_LJUMP:         u8 = 0x73;  // (i32);
-// const OP_LJUMP_FALSE:   u8 = 0x74;  // (i32);
-// const OP_LJUMP_TRUE:    u8 = 0x75;  // (i32);
+const OP_JUMP_FALSE:    u8 = 0x71;  // (i16); [ cond ] => [ cond ]
+const OP_JUMP_TRUE:     u8 = 0x72;  // (i16); [ cond ] => [ cond ]
+const OP_PJMP_FALSE:    u8 = 0x73;  // (i16); [ cond ] => []
+const OP_PJMP_TRUE:     u8 = 0x74;  // (i16); [ cond ] => []
 
-const OP_POPJMP_FALSE:  u8 = 0x76;  // (i16); [ _ ] => []
-const OP_POPJMP_TRUE:   u8 = 0x77;  // (i16); [ _ ] => []
-// const OP_POPLJMP:       u8 = 0x78;  // (i32); [ _ ] => []
-// const OP_POPLJMP_FALSE: u8 = 0x79;  // (i32); [ _ ] => []
-// const OP_POPLJMP_TRUE:  u8 = 0x7A;  // (i32); [ _ ] => []
+const OP_LJUMP:         u8 = 0x75;  // (i32);
+const OP_LJUMP_FALSE:   u8 = 0x76;  // (i32); [ cond ] => [ cond ]
+const OP_LJUMP_TRUE:    u8 = 0x77;  // (i32); [ cond ] => [ cond ]
+const OP_PLJMP_FALSE:   u8 = 0x78;  // (i32); [ cond ] => []
+const OP_PLJMP_TRUE:    u8 = 0x79;  // (i32); [ cond ] => []
 
+// 0x80-8F      Iteration
+
+// const OP_IT_INIT   // replace value with iterator state
+// const OP_IT_NEXT   // replace iterator state with next state
+// const OP_UNPACK:   // [ seq ] => [ item_0 ... item_n N ] -- used by unpack syntax in function calls
 
 
 // 0xF0         Debugging/Tracing/Misc
@@ -115,15 +125,19 @@ const DBG_DUMP_STRINGS: u8 = 0xF4;
 
 
 #[repr(u8)]
-#[derive(Clone, Copy, Hash, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq)]
 pub enum OpCode {
     Nop = OP_NOP,
     Return = OP_RETURN, 
+    Call = OP_CALL,
     
     Pop = OP_POP,
     Drop = OP_DROP,
     Clone = OP_CLONE,
     
+    Tuple = OP_TUPLE,
+    TupleN = OP_TUPLEN,
+    
     LoadConst  = OP_LD_CONST,
     LoadConst16 = OP_LD_CONST_16,
     InsertGlobal = OP_IN_GLOBAL_IM,
@@ -138,14 +152,14 @@ pub enum OpCode {
     LoadLocal16 = OP_LD_LOCAL_16,
     DropLocals = OP_DP_LOCALS,
     
-    Nil = OP_NIL,
-    True = OP_TRUE,
-    False = OP_FALSE,
-    Empty = OP_EMPTY,
-    Tuple = OP_TUPLE,
-    UInt8 = OP_U8,
-    Int8 = OP_I8,
-    Float8 = OP_F8,
+    Nil = OP_LD_NIL,
+    True = OP_LD_TRUE,
+    False = OP_LD_FALSE,
+    Empty = OP_LD_EMPTY,
+    
+    UInt8 = OP_LD_U8,
+    Int8 = OP_LD_I8,
+    Float8 = OP_LD_F8,
     
     Neg = OP_NEG,
     Pos = OP_POS,
@@ -172,8 +186,14 @@ pub enum OpCode {
     Jump = OP_JUMP,
     JumpIfFalse = OP_JUMP_FALSE,
     JumpIfTrue = OP_JUMP_TRUE,
-    PopJumpIfFalse = OP_POPJMP_FALSE,
-    PopJumpIfTrue = OP_POPJMP_TRUE,
+    PopJumpIfFalse = OP_PJMP_FALSE,
+    PopJumpIfTrue = OP_PJMP_TRUE,
+    
+    LongJump = OP_LJUMP,
+    LongJumpIfFalse = OP_LJUMP_FALSE,
+    LongJumpIfTrue = OP_LJUMP_TRUE,
+    PopLongJumpIfFalse = OP_PLJMP_FALSE,
+    PopLongJumpIfTrue = OP_PLJMP_TRUE,
     
     Inspect = DBG_INSPECT,
     Assert = DBG_ASSERT,
@@ -185,11 +205,15 @@ impl OpCode {
         let opcode = match byte {
             OP_NOP => Self::Nop,
             OP_RETURN => Self::Return,
+            OP_CALL => Self::Call,
             
             OP_POP => Self::Pop,
             OP_DROP => Self::Drop,
             OP_CLONE => Self::Clone,
             
+            OP_TUPLE => Self::Tuple,
+            OP_TUPLEN => Self::TupleN,
+            
             OP_LD_CONST => Self::LoadConst,
             OP_LD_CONST_16 => Self::LoadConst16,
             
@@ -205,14 +229,13 @@ impl OpCode {
             OP_LD_LOCAL_16 => Self::LoadLocal16,
             OP_DP_LOCALS => Self::DropLocals,
             
-            OP_NIL => Self::Nil,
-            OP_TRUE => Self::True,
-            OP_FALSE => Self::False,
-            OP_EMPTY => Self::Empty,
-            OP_TUPLE => Self::Tuple,
-            OP_U8 => Self::UInt8,
-            OP_I8 => Self::Int8,
-            OP_F8 => Self::Float8,
+            OP_LD_NIL => Self::Nil,
+            OP_LD_TRUE => Self::True,
+            OP_LD_FALSE => Self::False,
+            OP_LD_EMPTY => Self::Empty,
+            OP_LD_U8 => Self::UInt8,
+            OP_LD_I8 => Self::Int8,
+            OP_LD_F8 => Self::Float8,
             
             OP_NEG => Self::Neg,
             OP_POS => Self::Pos,
@@ -239,8 +262,14 @@ impl OpCode {
             OP_JUMP => Self::Jump,
             OP_JUMP_FALSE => Self::JumpIfFalse,
             OP_JUMP_TRUE => Self::JumpIfTrue,
-            OP_POPJMP_FALSE => Self::PopJumpIfFalse,
-            OP_POPJMP_TRUE => Self::PopJumpIfTrue,
+            OP_PJMP_FALSE => Self::PopJumpIfFalse,
+            OP_PJMP_TRUE => Self::PopJumpIfTrue,
+            
+            OP_LJUMP => Self::LongJump,
+            OP_LJUMP_FALSE => Self::LongJumpIfFalse,
+            OP_LJUMP_TRUE => Self::LongJumpIfTrue,
+            OP_PLJMP_FALSE => Self::PopLongJumpIfFalse,
+            OP_PLJMP_TRUE => Self::PopLongJumpIfTrue,
             
             DBG_INSPECT => Self::Inspect,
             DBG_ASSERT => Self::Assert,
@@ -251,9 +280,10 @@ impl OpCode {
     }
     
     #[inline]
-    pub fn instr_len(&self) -> usize {
+    pub const fn instr_len(&self) -> usize {
         match self {
             // don't really need size_of() for most of these, but it's a nice little bit of self-documentation
+            Self::Call           => 1 + size_of::<u8>(),
             
             Self::Drop           => 1 + size_of::<u8>(),
             
@@ -296,11 +326,15 @@ impl std::fmt::Display for OpCode {
         let mnemonic = match *self {
             Self::Nop => "OP_NOP",
             Self::Return => "OP_RETURN",
+            Self::Call => "OP_CALL",
             
             Self::Pop => "OP_POP",
             Self::Drop => "OP_DROP",
             Self::Clone => "OP_CLONE",
             
+            Self::Tuple => "OP_TUPLE",
+            Self::TupleN => "OP_TUPLEN",
+            
             Self::LoadConst => "OP_LD_CONST",
             Self::LoadConst16 => "OP_LD_CONST_16",
             
@@ -316,14 +350,13 @@ impl std::fmt::Display for OpCode {
             Self::LoadLocal16 => "OP_LD_LOCAL_16",
             Self::DropLocals => "OP_DP_LOCALS",
             
-            Self::Nil => "OP_NIL",
-            Self::True => "OP_TRUE",
-            Self::False => "OP_FALSE",
-            Self::Empty => "OP_EMPTY",
-            Self::Tuple => "OP_TUPLE",
-            Self::UInt8 => "OP_U8",
-            Self::Int8 => "OP_I8",
-            Self::Float8 => "OP_F8",
+            Self::Nil => "OP_LD_NIL",
+            Self::True => "OP_LD_TRUE",
+            Self::False => "OP_LD_FALSE",
+            Self::Empty => "OP_LD_EMPTY",
+            Self::UInt8 => "OP_LD_U8",
+            Self::Int8 => "OP_LD_I8",
+            Self::Float8 => "OP_LD_F8",
             
             Self::Neg => "OP_NEG",
             Self::Pos => "OP_POS",
@@ -350,8 +383,14 @@ impl std::fmt::Display for OpCode {
             Self::Jump => "OP_JUMP",
             Self::JumpIfFalse => "OP_JUMP_FALSE",
             Self::JumpIfTrue => "OP_JUMP_TRUE",
-            Self::PopJumpIfFalse => "OP_POPJMP_FALSE",
-            Self::PopJumpIfTrue => "OP_POPJMP_TRUE",
+            Self::PopJumpIfFalse => "OP_PJMP_FALSE",
+            Self::PopJumpIfTrue => "OP_PJMP_TRUE",
+            
+            Self::LongJump => "OP_LJUMP",
+            Self::LongJumpIfFalse => "OP_LJUMP_FALSE",
+            Self::LongJumpIfTrue => "OP_LJUMP_TRUE",
+            Self::PopLongJumpIfFalse => "OP_PLJMP_FALSE",
+            Self::PopLongJumpIfTrue => "OP_PLJMP_TRUE",
             
             Self::Inspect => "DBG_INSPECT",
             Self::Assert => "DBG_ASSERT",
