@@ -1,19 +1,20 @@
 use std::fmt;
 use std::fmt::{Write, Formatter};
 use std::iter;
+use std::collections::HashMap;
 use string_interner::Symbol as _;
 
 use crate::language::FloatType;
 use crate::codegen::OpCode;
-use crate::codegen::chunk::{UnloadedChunk, Constant, ConstID};
+use crate::codegen::chunk::{UnloadedProgram, Constant, ConstID, ChunkID};
 use crate::debug::symbol::{DebugSymbol, ResolvedSymbol, ResolvedSymbolTable, SymbolResolutionError};
 
 
 const PAD_WIDTH: usize = 60;
 
 pub struct Disassembler<'c, 's> {
-    chunk: &'c UnloadedChunk,
-    symbols: Option<&'s DebugSymbols>,
+    program: &'c UnloadedProgram,
+    symbols: Option<&'s ChunkSymbols>,
     symbol_table: Option<&'s ResolvedSymbolTable<'s>>,
 }
 
@@ -26,11 +27,11 @@ enum Symbol<'s> {
 }
 
 impl<'c, 's> Disassembler<'c, 's> {
-    pub fn new(chunk: &'c UnloadedChunk) -> Self {
-        Self { chunk, symbols: None, symbol_table: None }
+    pub fn new(program: &'c UnloadedProgram) -> Self {
+        Self { program, symbols: None, symbol_table: None }
     }
     
-    pub fn with_symbols(mut self, symbols: &'s DebugSymbols) -> Self {
+    pub fn with_symbols(mut self, symbols: &'s ChunkSymbols) -> Self {
         self.symbols.replace(symbols); self
     }
     
@@ -38,16 +39,31 @@ impl<'c, 's> Disassembler<'c, 's> {
         self.symbol_table.replace(symbol_table); self
     }
     
-    fn decode_chunk(&self, fmt: &mut Formatter<'_>) -> fmt::Result {
-        let mut symbols = self.symbols.map(|symbols| symbols.iter());
+    pub fn write_disassembly(&self, fmt: &mut impl Write) -> fmt::Result {
+        writeln!(fmt, "\n\nmain:\n")?;
+        let symbols = self.symbols.map(|symbols| symbols.get(&None)).flatten();
+        self.decode_chunk(fmt, self.program.main(), symbols)?;
+        
+        for (chunk_id, chunk) in self.program.iter_chunks() {
+            writeln!(fmt, "\n\nchunk {}:\n", chunk_id)?;
+            
+            let symbols = self.symbols.map(|symbols| symbols.get(&Some(chunk_id))).flatten();
+            self.decode_chunk(fmt, chunk, symbols)?;
+        }
+        
+        Ok(())
+    }
+    
+    fn decode_chunk(&self, fmt: &mut impl Write, chunk: &[u8], symbols: Option<&DebugSymbolsRLE>) -> fmt::Result {
+        let mut symbols = symbols.map(|symbols| symbols.iter());
         let mut last_symbol = None;
         
         let mut offset = 0;
-        while offset < self.chunk.bytes().len() {
-            let (_, bytes) = self.chunk.bytes().split_at(offset);
+        while offset < chunk.len() {
+            let (_, bytes) = chunk.split_at(offset);
             
             // get the next unresolved symbol if there are any
-            let unresolved = symbols.as_mut().map_or(None, |iter| iter.next());
+            let unresolved = symbols.as_mut().map_or(None, |iter| iter.next().flatten());
             let symbol = self.try_resolve_symbol(unresolved, last_symbol);
             last_symbol = unresolved;
             
@@ -74,7 +90,7 @@ impl<'c, 's> Disassembler<'c, 's> {
         } else { None }
     }
 
-    fn decode_instr(&self, fmt: &mut Formatter<'_>, offset: &usize, instr: &[u8], symbol: Option<Symbol>) -> Result<usize, fmt::Error> {        let mut line = String::new();
+    fn decode_instr(&self, fmt: &mut impl Write, offset: &usize, instr: &[u8], symbol: Option<Symbol>) -> Result<usize, fmt::Error> {        let mut line = String::new();
         
         write!(line, "{:04X} ", offset)?;
         
@@ -83,6 +99,11 @@ impl<'c, 's> Disassembler<'c, 's> {
         match opcode {
             Some(opcode) => match opcode {
                 
+                OpCode::Call => {
+                    let nargs = instr[1];
+                    write!(line, "{:16} {: >4}    ", opcode, nargs)?;
+                },
+                
                 OpCode::Drop | OpCode::DropLocals => {
                     let len = instr[1];
                     write!(line, "{:16} {: >4}    ", opcode, len)?;
@@ -91,13 +112,13 @@ impl<'c, 's> Disassembler<'c, 's> {
                 OpCode::LoadConst => {
                     let cid = instr[1];
                     write!(line, "{:16} {: >4}    ", opcode, cid)?;
-                    self.write_const(&mut line, self.chunk.lookup_const(cid))?;
+                    self.write_const(&mut line, self.program.lookup_const(cid))?;
                 },
                 
                 OpCode::LoadConst16 => {
                     let cid =  ConstID::from_le_bytes(instr[1..=2].try_into().unwrap());
                     write!(line, "{:16} {: >4}    ", opcode, cid)?;
-                    self.write_const(&mut line, self.chunk.lookup_const(cid))?;
+                    self.write_const(&mut line, self.program.lookup_const(cid))?;
                 },
                 
                 OpCode::StoreLocal | OpCode::LoadLocal => {
@@ -139,7 +160,17 @@ impl<'c, 's> Disassembler<'c, 's> {
                 OpCode::PopJumpIfFalse |
                 OpCode::PopJumpIfTrue  => {
                     let jmp = i16::from_le_bytes(instr[1..=2].try_into().unwrap());
-                    let dest = i64::from(jmp) + i64::try_from(offset + opcode.instr_len()).expect("offset too large");
+                    let dest = i128::from(jmp) + i128::try_from(offset + opcode.instr_len()).expect("offset too large");
+                    write!(line, "{:16} {: >4} -> {:04X}", opcode, jmp, dest)?;
+                }
+                
+                OpCode::LongJump           |
+                OpCode::LongJumpIfFalse    |
+                OpCode::LongJumpIfTrue     |
+                OpCode::PopLongJumpIfFalse |
+                OpCode::PopLongJumpIfTrue  => {
+                    let jmp = i32::from_le_bytes(instr[1..=4].try_into().unwrap());
+                    let dest = i128::from(jmp) + i128::try_from(offset + opcode.instr_len()).expect("offset too large");
                     write!(line, "{:16} {: >4} -> {:04X}", opcode, jmp, dest)?;
                 }
                 
@@ -195,13 +226,12 @@ impl<'c, 's> Disassembler<'c, 's> {
     }
     
     fn write_const(&self, fmt: &mut impl fmt::Write, value: &Constant) -> fmt::Result {
-        if let Constant::String(symbol) = value {
-            if let Some(string) = self.chunk.strings().resolve(*symbol) {
-                if string.len() > 16 {
-                    return write!(fmt, "\"{}...\"", &string[..13]);
-                }
-                return write!(fmt, "\"{}\"", string);
+        if let Constant::String(index) = value {
+            let string = self.program.get_string(*index);
+            if string.len() > 16 {
+                return write!(fmt, "\"{}...\"", &string[..13]);
             }
+            return write!(fmt, "\"{}\"", string);
         }
         
         write!(fmt, "{}", value)
@@ -210,7 +240,7 @@ impl<'c, 's> Disassembler<'c, 's> {
 
 impl fmt::Display for Disassembler<'_, '_> {
     fn fmt(&self, fmt: &mut Formatter<'_>) -> fmt::Result {
-        self.decode_chunk(fmt)
+        self.write_disassembly(fmt)
     }
 }
 
@@ -221,38 +251,81 @@ impl fmt::Display for Constant {
             Self::Integer(value) => write!(fmt, "'{}'", value),
             Self::Float(bytes) => write!(fmt, "'{:.6}'", FloatType::from_le_bytes(*bytes)),
             Self::String(symbol) => write!(fmt, "${}", symbol.to_usize() + 1),
+            Self::Function(chunk_id, _) => write!(fmt, "fun #{}", chunk_id),
         }
     }
 }
 
 
+pub type ChunkSymbols = HashMap<Option<ChunkID>, DebugSymbolsRLE>;
+
 // Container for debug symbols generated for bytecode
 // Should contain a DebugSymbol for each opcode in the 
 // associated Chunk, and in the same order.
 #[derive(Debug, Default, Clone)]
-pub struct DebugSymbols {
-    symbols: Vec<(DebugSymbol, u8)>,  // run length encoding
+pub struct DebugSymbolsRLE {
+    symbols: Vec<(Option<DebugSymbol>, u8)>,  // run length encoding
 }
 
-impl DebugSymbols {
+impl DebugSymbolsRLE {
     pub fn new() -> Self {
-        DebugSymbols {
+        Self {
             symbols: Vec::default(),
         }
     }
     
-    pub fn iter(&self) -> impl Iterator<Item=&DebugSymbol> { 
-        self.symbols.iter().flat_map(
-            |(sym, count)| std::iter::repeat(sym).take(usize::from(*count))
-        )
-    }
-    
-    pub fn push(&mut self, symbol: &DebugSymbol) {
+    pub fn push(&mut self, symbol: Option<DebugSymbol>) {
         match self.symbols.last_mut() {
-            Some((last, ref mut count)) if last == symbol && *count < u8::MAX => { 
+            Some((last, ref mut count)) if *last == symbol && *count < u8::MAX => { 
                 *count += 1 
             },
-            _ => { self.symbols.push((*symbol, 1)) }
+            _ => { self.symbols.push((symbol, 0)) }
         }
     }
-}
\ No newline at end of file
+    
+    pub fn iter(&self) -> impl Iterator<Item=Option<&DebugSymbol>> { 
+        self.symbols.iter().flat_map(
+            |(sym, count)| std::iter::repeat(sym.as_ref()).take(usize::from(*count) + 1)
+        )
+    }
+}
+
+
+#[derive(Debug, Default, Clone)]
+struct OffsetSpan {
+    offset: usize,
+    length: usize,
+}
+
+impl OffsetSpan {
+    pub fn start_offset(&self) -> usize { self.offset }
+    pub fn end_offset(&self) -> usize { self.offset + self.length }
+}
+
+
+#[derive(Debug, Default, Clone)]
+pub struct DebugSymbolOffsets {
+    symbols: Vec<(DebugSymbol, OffsetSpan)>,  // (symbol, offset) pairs
+}
+
+impl DebugSymbolOffsets {
+    pub fn new() -> Self {
+        Self {
+            symbols: Vec::default(),
+        }
+    }
+    
+    pub fn push(&mut self, symbol: DebugSymbol, offset: usize, length: usize) {
+        match self.symbols.last_mut() {
+            Some((_, ref mut span)) if span.end_offset() > offset => {
+                panic!("symbol inserted out of order");
+            }
+            
+            Some((last, ref mut span)) if *last == symbol && span.end_offset() == offset => { 
+                (*span).length += length
+            },
+            
+            _ => { self.symbols.push((symbol, OffsetSpan { offset, length })) }
+        }
+    }
+}
