@@ -38,11 +38,11 @@ use crate::runtime::opcodes::OpCode;
 fn dasm_test_opcode_const() {
     let mut chunk = Chunk::new();
     
-    let id = chunk.push_const(Variant::Float(1.2));
+    let id = chunk.get_or_insert_const(Variant::Float(1.2));
     chunk.push_byte(OpCode::LoadConst);
     chunk.push_byte(id as u8);
     
-    let id = chunk.push_const(Variant::Integer(0xDECA));
+    let id = chunk.get_or_insert_const(Variant::Integer(0xDECA));
     chunk.push_byte(OpCode::LoadConstWide);
     chunk.extend_bytes(&id.to_le_bytes());
     
