@@ -10,22 +10,20 @@ use crate::debug::SourceError;
 pub mod expr;
 pub mod stmt;
 pub mod primary;
-pub mod assign;
+pub mod lvalue;
 pub mod operator;
 pub mod fundefs;
-pub mod structs;
 pub mod errors;
 mod tests;
 
 pub use errors::{ParserError, ParseResult};
 
-use expr::{ExprMeta, Expr, Conditional, CondBranch};
+use expr::{ExprMeta, Expr, ExprBlock, ConditionalBranch};
 use stmt::{StmtMeta, StmtList, Stmt, Label, ControlFlow};
-use primary::{Primary, Atom, AccessItem};
-use assign::{Assignment, LValue, Declaration, DeclType};
+use primary::{Primary, Atom, AccessItem, Argument};
+use lvalue::{Assignment, LValue, Declaration, DeclType};
 use operator::{UnaryOp, BinaryOp, Precedence, PRECEDENCE_START, PRECEDENCE_END};
-use fundefs::{FunctionDef, FunSignature, FunParam};
-use structs::{ObjectConstructor};
+use fundefs::{FunctionDef, SignatureDef, ParamDef, DefaultDef};
 use errors::{ErrorKind, ErrorContext, ContextTag};
 
 
@@ -224,18 +222,22 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         
         ctx.push(ContextTag::StmtMeta);
         
-        let stmt = self.parse_toplevel_stmt(ctx)?;
+        let stmt = self.parse_stmt_variant(ctx)?;
         let symbol = ctx.frame().as_debug_symbol().unwrap();
         
         ctx.pop_extend();
         Ok(StmtMeta::new(stmt, symbol))
     }
     
-    fn parse_toplevel_stmt(&mut self, ctx: &mut ErrorContext) -> ParseResult<Stmt> {
+    fn parse_stmt_variant(&mut self, ctx: &mut ErrorContext) -> ParseResult<Stmt> {
         let stmt = match  self.peek()?.token {
-            Token::While => unimplemented!(),
-            Token::Do => unimplemented!(),
+            
+            Token::Loop => self.parse_loop(ctx, None)?,
+            Token::While => self.parse_while_loop(ctx, None)?,
             Token::For => unimplemented!(),
+            
+            Token::Label(..) => self.parse_stmt_label(ctx)?,
+            
             Token::Echo => {
                 ctx.set_start(&self.advance().unwrap());
                 Stmt::Echo(self.parse_expr_variant(ctx)?)
@@ -246,9 +248,9 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             }
             
             Token::Continue | Token::Break | Token::Return => {
-                ctx.push(ContextTag::ControlFlow);
-                
                 let next = self.advance().unwrap();
+                
+                ctx.push(ContextTag::ControlFlow);
                 ctx.set_start(&next);
                 
                 let name = match next.token {
@@ -267,7 +269,59 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         Ok(stmt)
     }
     
-    // parses a list of statements, stopping when the given closure returns true. The final token is not consumed.
+    fn parse_stmt_label(&mut self, ctx: &mut ErrorContext) -> ParseResult<Stmt> {
+        let label = self.try_parse_label(ctx)?.unwrap();
+        
+        match self.peek()?.token {
+            Token::Loop => self.parse_loop(ctx, Some(label)),
+            Token::While => self.parse_while_loop(ctx, Some(label)),
+            Token::For => unimplemented!(),
+            
+            _ => return Err("labels must be followed by either a block or a loop".into()),
+        }
+    }
+    
+    fn parse_loop(&mut self, ctx: &mut ErrorContext, label: Option<Label>) -> ParseResult<Stmt> {
+        let next = self.advance()?;
+        
+        ctx.push(ContextTag::Loop);
+        ctx.set_start(&next);
+        debug_assert!(matches!(next.token, Token::Loop));
+        
+        let body = self.parse_stmt_list(ctx, |token| matches!(token, Token::End))?;
+        
+        let next = self.advance().unwrap(); 
+        ctx.set_end(&next);
+        
+        ctx.pop_extend();
+        Ok(Stmt::Loop { label, body })
+    }
+    
+    fn parse_while_loop(&mut self, ctx: &mut ErrorContext, label: Option<Label>) -> ParseResult<Stmt> {
+        let next = self.advance()?;
+        
+        ctx.push(ContextTag::WhileLoop);
+        ctx.set_start(&next);
+        debug_assert!(matches!(next.token, Token::While));
+        
+        let condition = self.parse_expr_variant(ctx)?;
+        
+        let next = self.advance()?;
+        ctx.set_end(&next);
+        
+        if !matches!(next.token, Token::Do) {
+            return Err("missing \"do\" after condition in while-loop".into());
+        }
+        
+        let body = self.parse_stmt_list(ctx, |token| matches!(token, Token::End))?;
+        
+        ctx.set_end(&self.advance().unwrap()); // consume "end"
+        
+        ctx.pop_extend();
+        Ok(Stmt::WhileLoop { label, condition, body })
+    }
+    
+    /// Parses a list of statements, stopping when the given closure returns true. The final token is not consumed.
     fn parse_stmt_list(&mut self, ctx: &mut ErrorContext, end_list: impl Fn(&Token) -> bool) -> ParseResult<StmtList> {
         ctx.push(ContextTag::StmtList);
         
@@ -278,18 +332,23 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         
         loop {
             
+            // statement separators
+            while matches!(self.peek()?.token, Token::Semicolon) {
+                ctx.set_end(&self.advance().unwrap());
+            }
+            
             let next = self.peek()?;
             if end_list(&next.token) {
                 break;
             }
             
             let parse_result = self.try_parse_control_flow(ctx);
-            
             control = match self.catch_error_and_sync(ctx, parse_result, true) {
                 Some(result) => result?,
                 None => continue,
             };
             
+            // if we found a control flow statement, it must be the last item in the statement list
             if let Some(control) = control.as_ref() {
                 
                 let next = self.peek()?;
@@ -301,7 +360,6 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                         ControlFlow::Continue(..) => "continue",
                         ControlFlow::Break(..) => "break",
                         ControlFlow::Return(..) => "return",
-                        _ => unreachable!(),
                     };
                     
                     let message = format!("\"{}\" must be the last statement in a block", name);
@@ -330,22 +388,6 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         Ok(StmtList::new(suite, control))
     }
     
-    // if the last statement is an expression statement it becomes the expression for the block
-    fn parse_stmt_list_expr(&mut self, ctx: &mut ErrorContext, end_list: impl Fn(&Token) -> bool) -> ParseResult<StmtList> {
-        let (mut suite, mut control_flow) = self.parse_stmt_list(ctx, end_list)?.take();
-        
-        if control_flow.is_none() && !suite.is_empty() {
-            match ExprMeta::try_from(suite.pop().unwrap()) {
-                Err(stmt) => suite.push(stmt),
-                Ok(expr) => {
-                    control_flow.replace(ControlFlow::Expression(Box::new(expr)));
-                }
-            }
-        }
-        
-        Ok(StmtList::new(suite, control_flow))
-    }
-    
     fn try_parse_label(&mut self, _ctx: &mut ErrorContext) -> ParseResult<Option<Label>> {
         let next = self.peek()?;
         
@@ -402,8 +444,6 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             _ => return Ok(None),
         };
         
-        // let symbol = ctx.frame().as_debug_symbol().unwrap();
-        
         ctx.pop_extend();
         Ok(Some(control_flow))
     }
@@ -493,9 +533,9 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         declaration_expression ::= ( "let" | "var" ) lvalue_expr_annotated "=" expression ;
     */
     fn parse_declaration_expr(&mut self, ctx: &mut ErrorContext) -> ParseResult<Expr> {
-        ctx.push(ContextTag::DeclarationExpr);
-        
         let next = self.advance()?;
+        
+        ctx.push(ContextTag::DeclarationExpr);
         ctx.set_start(&next);
         
         let decl = match next.token {
@@ -719,8 +759,11 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         let expr = match self.peek()?.token {
             Token::Class => unimplemented!(),
             Token::Fun => self.parse_function_decl_expr(ctx)?,
+            
             Token::If => self.parse_if_expr(ctx)?,
-            Token::Begin | Token::Label(..) => self.parse_block_expr(ctx)?,
+            Token::Begin => self.parse_block_expr(ctx, None)?,
+            
+            Token::Label(..) => self.parse_expr_label(ctx)?,
             
             // Token::OpenBrace => Ok(ExprMeta::ObjectCtor(self.parse_object_constructor(ctx)?)),
             
@@ -729,40 +772,46 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         Ok(expr)
     }
     
+    fn parse_expr_label(&mut self, ctx: &mut ErrorContext) -> ParseResult<Expr> {
+        let label = self.try_parse_label(ctx)?.unwrap();
+        
+        let expr = match self.peek()?.token {
+            Token::Begin => self.parse_block_expr(ctx, Some(label))?,
+            _ => return Err("labels must be followed by either a block or a loop".into()),
+        };
+        
+        Ok(expr)
+    }
+    
     /*
         block-expression ::= ( label )? "begin" ( statement | control-flow | "break" ( label )? expression )* "end" ;  (* break can be supplied a value inside of begin-blocks *)
     */
-    fn parse_block_expr(&mut self, ctx: &mut ErrorContext) -> ParseResult<Expr> {
-        ctx.push(ContextTag::BlockExpr);
-        ctx.set_start(self.peek()?);
-        
-        // check for label
-        let block_label = self.try_parse_label(ctx)?;
+    fn parse_block_expr(&mut self, ctx: &mut ErrorContext, label: Option<Label>) -> ParseResult<Expr> {
+        let next = self.advance()?;
         
         // consume "begin"
-        let next = self.advance()?;
-        ctx.set_end(&next);
-        if !matches!(next.token, Token::Begin) {
-            return Err("block labels must be followed by either a block or a loop".into());
-        }
+        ctx.push(ContextTag::BlockExpr);
+        ctx.set_start(&next);
+        debug_assert!(matches!(next.token, Token::Begin));
         
-        let stmt_list = self.parse_stmt_list_expr(ctx, |token| matches!(token, Token::End))?;
+        let suite = self.parse_stmt_list(ctx, |token| matches!(token, Token::End))?;
         ctx.set_end(&self.advance().unwrap()); // consume "end"
         
         ctx.pop_extend();
-        Ok(Expr::Block(block_label, stmt_list))
+        
+        Ok(Expr::Block { label, suite: Box::new(ExprBlock::from(suite)), })
     }
     
     fn parse_if_expr(&mut self, ctx: &mut ErrorContext) -> ParseResult<Expr> {
-        ctx.push(ContextTag::IfExpr);
-        
         let next = self.advance()?;
+        
+        ctx.push(ContextTag::IfExpr);
         ctx.set_start(&next);
         
         debug_assert!(matches!(next.token, Token::If));
         
         let mut branches = Vec::new();
-        let mut else_branch = None;
+        let mut else_clause = None;
         
         loop {
             
@@ -776,9 +825,9 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 return Err("missing \"then\" after condition in if-expression".into());
             }
             
-            let stmt_list = self.parse_stmt_list_expr(ctx, |token| matches!(token, Token::Elif | Token::Else | Token::End))?;
+            let stmt_list = self.parse_stmt_list(ctx, |token| matches!(token, Token::Elif | Token::Else | Token::End))?;
             
-            branches.push(CondBranch::new(cond_expr, stmt_list));
+            branches.push(ConditionalBranch::new(cond_expr, stmt_list.into()));
             
             let next = self.advance().unwrap();
             ctx.set_end(&next);
@@ -787,8 +836,8 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 Token::End => break,
                 Token::Else => {
                     
-                    let stmt_list = self.parse_stmt_list_expr(ctx, |token| matches!(token, Token::End))?;
-                    else_branch.replace(stmt_list);
+                    let stmt_list = self.parse_stmt_list(ctx, |token| matches!(token, Token::End))?;
+                    else_clause.replace(ExprBlock::from(stmt_list));
                     
                     ctx.set_end(&self.advance().unwrap()); // consume "end"
                     
@@ -802,13 +851,17 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         
         ctx.pop_extend();
         
-        Ok(Expr::IfExpr(Conditional::new(branches, else_branch)))
+        let if_expr = Expr::IfExpr { 
+            branches: branches.into_boxed_slice(),
+            else_clause: else_clause.map(|expr| Box::new(expr)),
+        };
+        Ok(if_expr)
     }
     
     fn parse_function_decl_expr(&mut self, ctx: &mut ErrorContext) -> ParseResult<Expr> {
-        ctx.push(ContextTag::FunDefExpr);
-        
         let next = self.advance()?;
+        
+        ctx.push(ContextTag::FunDefExpr);
         ctx.set_start(&next);
         debug_assert!(matches!(next.token, Token::Fun));
         
@@ -837,19 +890,37 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         }
     }
     
-    fn _parse_anonymous_function(&mut self, ctx: &mut ErrorContext) -> ParseResult<Expr> {
-        ctx.push(ContextTag::FunDefExpr);
+    // similar to parse_primary(), except we only allow member access and index access, and convert to an LValue after
+    fn parse_function_assignment_target(&mut self, ctx: &mut ErrorContext) -> ParseResult<LValue> {
+        ctx.push(ContextTag::PrimaryExpr);
         
-        let next = self.advance()?;
-        ctx.set_start(&next);
-        debug_assert!(matches!(next.token, Token::Fun));
+        let atom = self.parse_atom(ctx)?;
         
-        let func_def = self.parse_function_def(ctx)?;
+        let mut items = Vec::new();
+        loop {
+            let next = self.peek()?;
+            match next.token {
+                
+                // access ::= "." IDENTIFIER ;
+                Token::OpAccess => items.push(self.parse_member_access(ctx)?),
+                
+                // subscript ::= "[" expression "]" ;
+                Token::OpenSquare => items.push(self.parse_index_access(ctx)?),
+                
+                _ => break,
+            };
+        }
         
         ctx.pop_extend();
-        Ok(Expr::FunctionDef(func_def))
-    }
         
+        let lvalue =
+            if items.is_empty() { LValue::try_from(atom) } 
+            else { LValue::try_from(Primary::new(atom, items)) }
+            .map_err(|_| ParserError::from("cannot assign a function to this"))?;
+        
+        Ok(lvalue)
+    }
+    
     fn parse_function_def(&mut self, ctx: &mut ErrorContext) -> ParseResult<FunctionDef> {
         // expect open paren now
         let next = self.advance().unwrap();
@@ -873,10 +944,15 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         let body = self.parse_stmt_list(ctx, |token| matches!(token, Token::End))?;
         ctx.set_end(&self.advance().unwrap()); // consume "end"
         
-        Ok(FunctionDef::new(signature, body))
+        let fundef = FunctionDef {
+            signature,
+            body: Box::new(ExprBlock::from(body)),
+        };
+        
+        Ok(fundef)
     }
     
-    fn parse_function_param_list(&mut self, ctx: &mut ErrorContext) -> ParseResult<FunSignature> {
+    fn parse_function_param_list(&mut self, ctx: &mut ErrorContext) -> ParseResult<SignatureDef> {
 
         let mut required = Vec::new();
         let mut default = Vec::new();
@@ -928,38 +1004,42 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             
             // possible default value
             let next = self.peek()?;
-            let default_value =
-                if matches!(next.token, Token::OpAssign) {
+            let default_value = match next.token {
+                Token::OpAssign if is_variadic => {
+                    return Err("variadic parameters can't have a default value".into())
+                },
+                
+                Token::OpAssign => {
                     ctx.set_end(&self.advance().unwrap());
                     
-                    Some(self.parse_expr_variant(ctx)?)
-                } else {
-                    None
-                };
+                    Some(Box::new(self.parse_expr(ctx)?))
+                },
+                
+                _ => None,
+            };
             
             // expect either a comma "," or the closing ")"
             let next = self.peek()?;
             match next.token {
                 // variadic parameter
                 Token::Comma if is_variadic => {
-                    return Err("a variadic parameter must be the last one in the parameter list".into());
+                    return Err("a variadic parameter must appear last in the parameter list".into());
                 }
+                
                 Token::CloseParen if is_variadic => {
-                    variadic.replace(FunParam::new(name, decl, default_value));
+                    debug_assert!(default_value.is_none());
+                    variadic.replace(ParamDef { name, decl });
                 },
                 
                 // normal parameter
                 Token::Comma | Token::CloseParen if !is_variadic => {
-                    let has_default = default_value.is_some();
-                    
-                    let param = FunParam::new(name, decl, default_value);
-                    if has_default {
-                        default.push(param);
+                    if default_value.is_some() {
+                        default.push(DefaultDef { name, decl, default: default_value.unwrap() });
                     } else {
                         if !default.is_empty() {
                             return Err("cannot have a non-default parameter after a default parameter".into());
                         }
-                        required.push(param);
+                        required.push(ParamDef { name, decl });
                     }
                 },
                 
@@ -973,7 +1053,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             ctx.pop_extend();
         }
         
-        Ok(FunSignature::new(required, default, variadic))
+        Ok(SignatureDef::new(required, default, variadic))
     }
     
     /*
@@ -983,6 +1063,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         member-initializer ::= ( IDENTIFIER | "[" primary "]" ) ":" expression ;
     
     */
+    /*
     fn parse_object_constructor(&mut self, ctx: &mut ErrorContext) -> ParseResult<ObjectConstructor> {
         ctx.push(ContextTag::ObjectCtor);
         
@@ -1001,6 +1082,7 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         
         // ctx.pop_extend();
     }
+    */
     
     /*
         Primary expression syntax:
@@ -1027,15 +1109,12 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
                 // subscript ::= "[" expression "]" ;
                 Token::OpenSquare => items.push(self.parse_index_access(ctx)?),
                 
-                // Invocation is a special case. 
-                // The parens containing the argument list are not allowed to be on a separate line
-                // Lua has a similar issue, but doesn't have tuples so it isn't as big a problem there
-                // By checking next.newline we ensure that a () on the next line is properly parsed as a tuple
                 
-                // invocation ::= "(" ... ")" ;  (* WIP *)
-                Token::OpenParen if !next.newline => {
-                    unimplemented!()
-                }
+                // invocation ::= "(" ")" | "(" argument ( "," argument )* ")" ; 
+                // argument ::= expression ( "..." )? ;  (* "..." is for argument unpacking syntax *)
+                
+                // invocations are not allowed to be on a separate line from the invocation receiver
+                Token::OpenParen if !next.newline => items.push(self.parse_invocation(ctx)?),
                 
                 // object-constructor ::= "{" ... "}"
                 Token::OpenBrace => {
@@ -1070,41 +1149,13 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         }
     }
     
-    // similar to parse_primary(), except we only allow member access and index access, and convert to an LValue after
-    fn parse_function_assignment_target(&mut self, ctx: &mut ErrorContext) -> ParseResult<LValue> {
-        ctx.push(ContextTag::PrimaryExpr);
-        
-        let atom = self.parse_atom(ctx)?;
-        
-        let mut items = Vec::new();
-        loop {
-            let next = self.peek()?;
-            match next.token {
-                
-                // access ::= "." IDENTIFIER ;
-                Token::OpAccess => items.push(self.parse_member_access(ctx)?),
-                
-                // subscript ::= "[" expression "]" ;
-                Token::OpenSquare => items.push(self.parse_index_access(ctx)?),
-                
-                _ => break,
-            };
-        }
-        
-        ctx.pop_extend();
-        
-        let lvalue =
-            if items.is_empty() { LValue::try_from(atom) } 
-            else { LValue::try_from(Primary::new(atom, items)) }
-            .map_err(|_| ParserError::from("cannot assign a function to this"))?;
-        
-        Ok(lvalue)
-    }
-    
     // access ::= "." IDENTIFIER ;
     fn parse_member_access(&mut self, ctx: &mut ErrorContext) -> ParseResult<AccessItem> {
+        let next = self.advance().unwrap();
+        
         ctx.push(ContextTag::MemberAccess);
-        ctx.set_start(&self.advance().unwrap());
+        ctx.set_start(&next);
+        debug_assert!(matches!(next.token, Token::OpAccess));
         
         let next = self.advance()?;
         ctx.set_end(&next);
@@ -1122,23 +1173,57 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
     
     // subscript ::= "[" expression "]" ;
     fn parse_index_access(&mut self, ctx: &mut ErrorContext) -> ParseResult<AccessItem> {
+        let next = self.advance().unwrap();
+        
         ctx.push(ContextTag::IndexAccess);
-        ctx.set_start(&self.advance().unwrap());
+        ctx.set_start(&next);
+        debug_assert!(matches!(next.token, Token::OpenSquare));
         
         let index_expr = self.parse_expr(ctx)?;
         
         let next = self.advance()?;
         ctx.set_end(&next);
         
-        let item;
-        if matches!(next.token, Token::CloseSquare) {
-            item = AccessItem::Index(index_expr);
-        } else {
+        if !matches!(next.token, Token::CloseSquare) {
             return Err("missing closing \"]\"".into());
         }
         
         ctx.pop_extend();
-        Ok(item)
+        Ok(AccessItem::Index(index_expr))
+    }
+    
+    fn parse_invocation(&mut self, ctx: &mut ErrorContext) -> ParseResult<AccessItem> {
+        let next = self.advance().unwrap();
+        
+        ctx.push(ContextTag::Invocation);
+        ctx.set_start(&next);
+        debug_assert!(matches!(next.token, Token::OpenParen));
+        
+        let mut args = Vec::new();
+        
+        loop {
+            if matches!(self.peek()?.token, Token::CloseParen) {
+                break;
+            }
+            
+            let arg_expr = self.parse_expr(ctx)?;
+            if let Token::Ellipsis = self.peek()?.token {
+                ctx.set_end(&self.advance().unwrap());
+                
+                args.push(Argument::unpack(arg_expr));
+            } else { 
+                args.push(Argument::new(arg_expr));
+            }
+            
+            match self.peek()?.token {
+                Token::Comma => ctx.set_end(&self.advance().unwrap()),
+                Token::CloseParen => break,
+                _ => return Err("missing \",\" between invocation arguments".into()),
+            }
+        }
+        
+        ctx.pop_extend();
+        Ok(AccessItem::Invoke(args.into_boxed_slice()))
     }
     
     // atom ::= LITERAL | IDENTIFIER | "(" expression ")" ;
