@@ -10,7 +10,7 @@ use crate::runtime::DefaultBuildHasher;
 
 
 thread_local! {
-    pub static STRING_TABLE: StringTable = StringTable::new();
+    pub static STRING_TABLE: RefCell<StringTable> = RefCell::new(StringTable::new());
 }
 
 
@@ -31,10 +31,22 @@ impl StringSymbol {
     }
     
     pub fn intern(string: &str) -> Self {
-        STRING_TABLE.with(|str_tbl| str_tbl.get_or_intern(string))
+        STRING_TABLE.with(|string_table| string_table.borrow_mut().get_or_intern(string))
+    }
+    
+    pub fn concat(&self, other: &StringSymbol) -> StringSymbol {
+        STRING_TABLE.with(|string_table| {
+            let mut string_table = string_table.borrow_mut();
+            
+            let mut buf = String::new();
+            buf.push_str(string_table.resolve(self));
+            buf.push_str(string_table.resolve(other));
+            string_table.get_or_intern(buf.as_str())
+        })
     }
 }
 
+
 // not implementing Deref for StringSymbol because I don't want to hide the cost of acquiring a read lock
 
 impl From<StringSymbol> for InternSymbol {
@@ -52,10 +64,12 @@ impl From<InternSymbol> for StringSymbol {
 // Lexicographical ordering of strings
 impl PartialOrd for StringSymbol {
     fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
-        STRING_TABLE.with(|str_tbl| {
+        STRING_TABLE.with(|string_table| {
+            let string_table = string_table.borrow();
+            
             <str as PartialOrd>::partial_cmp(
-                &str_tbl.resolve(self),
-                &str_tbl.resolve(other),
+                string_table.resolve(self),
+                string_table.resolve(other),
             )
         })
 
@@ -64,10 +78,12 @@ impl PartialOrd for StringSymbol {
 
 impl Ord for StringSymbol {
     fn cmp(&self, other: &Self) -> cmp::Ordering {
-        STRING_TABLE.with(|str_tbl| {
+        STRING_TABLE.with(|string_table| {
+            let string_table = string_table.borrow();
+            
             <str as Ord>::cmp(
-                &str_tbl.resolve(self),
-                &str_tbl.resolve(other),
+                string_table.resolve(self),
+                string_table.resolve(other),
             )
         })
     }
@@ -75,7 +91,9 @@ impl Ord for StringSymbol {
 
 impl fmt::Display for StringSymbol {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
-        STRING_TABLE.with(|str_tbl| fmt.write_str(&str_tbl.resolve(self)))
+        STRING_TABLE.with(|string_table| fmt.write_str(
+            string_table.borrow().resolve(self)
+        ))
     }
 }
 
@@ -87,51 +105,40 @@ type InternBackend = DefaultBackend<DefaultSymbol>;
 // StringTable is used for string symbol lookups at runtime
 pub type StringInterner = string_interner::StringInterner<InternBackend, DefaultBuildHasher>;
 
-#[derive(Clone)]
+#[derive(Default, Clone)]
 pub struct StringTable {
-    interner: RefCell<StringInterner>,
+    interner: StringInterner,
 }
 
 impl StringTable {
     pub fn new() -> Self {
         StringTable {
-            interner: RefCell::new(StringInterner::new()),
+            interner: StringInterner::new(),
         }
     }
     
     pub fn get(&self, string: &str) -> Option<StringSymbol> {
-        self.interner.borrow_mut().get(string).map(|symbol| symbol.into())
+        self.interner.get(string).map(|symbol| symbol.into())
     }
     
-    pub fn get_or_intern(&self, string: &str) -> StringSymbol {
-        self.interner.borrow_mut().get_or_intern(string).into()
+    pub fn get_or_intern(&mut self, string: &str) -> StringSymbol {
+        self.interner.get_or_intern(string).into()
     }
     
-    pub fn resolve(&self, symbol: &StringSymbol) -> impl Deref<Target=str> + '_ {
+    pub fn resolve(&self, symbol: &StringSymbol) -> &str {
         let symbol = InternSymbol::from(*symbol);
-        Ref::map(self.interner.borrow(), |interner| interner.resolve(symbol).unwrap())
+        self.interner.resolve(symbol).expect("invalid symbol")
     }
     
-    // effectively consumes the string table, leaving it empty
-    pub fn take_interner(&self) -> StringInterner {
-        self.interner.take()
-    }
-
-    pub fn interner(&self) -> impl Deref<Target=StringInterner> + '_ {
-        self.interner.borrow()
-    }
-    
-    // helpful for interning a bunch of strings without the overhead of repeatedly borrowing
-    pub fn interner_mut(&self) -> impl DerefMut<Target=StringInterner> + '_ {
-        self.interner.borrow_mut()
-    }
+    // pub fn into_iter(&self) -> impl Iterator<Item=(StringSymbol, &str)> {
+    //     self.interner.into_iter().map(|(symbol, string)| (symbol.into(), string))
+    // }
 }
 
 impl<'s> Extend<&'s str> for StringTable {
     fn extend<T>(&mut self, iter: T) where T: IntoIterator<Item=&'s str> {
-        let mut interner = self.interner.borrow_mut();
         for string in iter.into_iter() {
-            interner.get_or_intern(string);
+            self.interner.get_or_intern(string);
         }
     }
 }
