@@ -6,26 +6,15 @@ use crate::language::{IntType, FloatType};
 use crate::runtime::types::Metatable;
 use crate::runtime::types::primitive::*;
 use crate::runtime::strings::{StringSymbol, STRING_TABLE};
+use crate::runtime::gc::GCHandle;
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
 
-// Temporary until a proper type system is implemented?
-#[derive(Clone, Copy, PartialEq, Eq, Hash)]
-pub enum TypeTag {
-    Nil,
-    Tuple,
-    Bool,
-    Integer,
-    Float,
-    String,
-    Object,
-}
-
 // Fundamental data value type
 #[derive(Clone)] // add Copy?
 pub enum Variant {
     Nil,
-    EmptyTuple, // the empty tuple value
+    EmptyTuple,
     BoolTrue,
     BoolFalse,
     Integer(IntType),
@@ -33,22 +22,11 @@ pub enum Variant {
     String(StringSymbol),
     // TODO just GC tuples
     Tuple(Rc<[Variant]>),  //  will use COW semantics, so if we need to send to another thread we can just clone the underlying data
-    //Object(GCHandle),
+    Object(GCHandle),
+    // LoadedModule(ModuleID)
 }
 
 impl Variant {
-    //TODO type system
-    pub fn type_tag(&self) -> TypeTag {
-        match self {
-            Self::Nil => TypeTag::Nil,
-            Self::BoolTrue | Self::BoolFalse => TypeTag::Bool,
-            Self::Integer(..) => TypeTag::Integer,
-            Self::Float(..) => TypeTag::Float,
-            Self::String(..) => TypeTag::String,
-            Self::EmptyTuple | Self::Tuple(..) => TypeTag::Tuple,
-        }
-    }
-    
     pub fn metatable(&self) -> &Metatable {
         match self {
             Self::String(..) => &METATABLE_STRING,
@@ -61,9 +39,6 @@ impl Variant {
         !matches!(self, Self::Nil | Self::BoolFalse)
     }
     
-    // Note, bit_value() and float_value() are defined based on what is needed for the language *implementation*
-    // They do not reflect the semantics of the Sphinx language
-    
     pub fn bit_value(&self) -> Option<IntType> {
         let value = match self {
             Self::Integer(value) => *value,
@@ -120,21 +95,10 @@ impl From<StringSymbol> for Variant {
     fn from(value: StringSymbol) -> Self { Variant::String(value) }
 }
 
-
-const AUTO_INTERN_MAX: usize = 40;
-
 impl From<&str> for Variant {
     fn from(value: &str) -> Self {
-        if value.len() <= AUTO_INTERN_MAX {
-            return STRING_TABLE.with(|string_table| string_table.get_or_intern(value)).into();
-        }
-        
-        if let Some(symbol) = STRING_TABLE.with(|string_table| string_table.get(value)) {
-            return symbol.into();
-        }
-        
-        // long strings
-        unimplemented!()
+        let symbol = STRING_TABLE.with(|string_table| string_table.borrow_mut().get_or_intern(value));
+        symbol.into()
     }
 }
 
@@ -223,6 +187,8 @@ impl fmt::Debug for Variant {
                 }
                 write!(fmt, "{:?})", last)
             }
+            
+            Self::Object(_handle) => unimplemented!(),
         }
     }
 }
