@@ -10,6 +10,7 @@ use crate::runtime::strings::StringInterner;
 
 type ReadFileChars = ReadChars<io::BufReader<fs::File>>;
 
+#[derive(Debug, Hash)]
 pub enum SourceType {
     String(String),
     File(PathBuf),
@@ -25,7 +26,7 @@ impl<S> From<S> for SourceText where S: ToString {
     fn from(text: S) -> Self { SourceText::String(text.to_string()) }
 }
 
-// Represents a "source" of source code, and provides the means to access the source text as a sequence of chars
+#[derive(Debug, Hash)]
 pub struct ModuleSource {
     name: String,
     source: SourceType,
